/* *******************************************************************************
 * 
 * Camouflage
 * ----------
 * 
 * cryptographic steganography for MP3 files
 * 
 * Copyright  2003-2004 Danilo Bzdok
 * 
 * Visit my digital home: http://sistemo.has.it
 * 
 * YOU HAVE TO ASK ME IF YOU WANT TO GAIN A PERMISSION TO IMPLEMENT AT LEAST
 * PARTS OF THIS CODE! ANY OTHER KIND OF REUSING, RECYCLING OR COPYING OF THIS
 * SOURCE CODE IS AGAINST THE LAW.
 * 
 *******************************************************************************/ 

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Threading;
using System.Text;
using System.IO;
using System.Runtime.InteropServices;

using camouflage.utils;

namespace camouflage
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class MainFrame : System.Windows.Forms.Form
	{
		//
		// some constants
		//
		string       currentLanguage               = "English";
		const string stdTitle                      = "[ Camouflage 1.0 ]";
		const uint   CF_PROLOG_SIZE_ASSUMPTION     = 100;

		private System.Windows.Forms.PictureBox pictureBox1;
		private System.Windows.Forms.GroupBox groupBox1;
		private System.Windows.Forms.GroupBox groupBox2;
		private System.Windows.Forms.Panel panWelcome;
		private System.Windows.Forms.LinkLabel linkLabel1;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private Panel[][] tourList;
		private int       tourIndex = 0;
		private int       bExtract;
		private System.Windows.Forms.Button butBack;
		private System.Windows.Forms.Button butCancel;
		private System.Windows.Forms.Button butNext;
		private System.Windows.Forms.ComboBox cbLangs;
		private System.Windows.Forms.Label labChooseLang;
		private System.Windows.Forms.GroupBox groupBox3;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Panel panWhatToDo;
		private System.Windows.Forms.RadioButton radEmbed;
		private System.Windows.Forms.RadioButton radExtract;
		private System.Windows.Forms.Label labWelcome;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.Panel panChooseFiles;
		private System.Windows.Forms.Label labChooseFiles;
		private System.Windows.Forms.Label labTotSecrets;
		private System.Windows.Forms.Label labTotSecretFSize;
		private System.Windows.Forms.ListView lvSecretFiles;
		private System.Windows.Forms.Button butAddSecret;
		private System.Windows.Forms.Button butDelSecret;
		private System.Windows.Forms.OpenFileDialog ofn;
		private utils.TextEnum Strings   = null;
		private long cbSecretFiles       = 0;
		private long cbHidable           = 0;
		private System.Windows.Forms.Panel panChooseMP3Emb;
		private System.Windows.Forms.Button butDelMP3Emb;
		private System.Windows.Forms.Button butAddMP3Emb;
		private System.Windows.Forms.ListView lvMP3sEmb;
		private System.Windows.Forms.Label labChooseMP3Emb;
		private System.Windows.Forms.Label labStep;
		private System.Windows.Forms.Label labData2Hide;
		private System.Windows.Forms.Label labSize2Hide;
		private System.Windows.Forms.Panel panPassword;
		private System.Windows.Forms.Label labEnterPW;
		private System.Windows.Forms.Label labPW1;
		private System.Windows.Forms.TextBox txtPW1;
		private System.Windows.Forms.TextBox txtPW2;
		private System.Windows.Forms.Label labPW2;
		private System.Windows.Forms.Panel panEmbEnd;
		private System.Windows.Forms.Label llabMoreMp3s;
		private System.Windows.Forms.Label labEmbEnd;
		private System.Windows.Forms.ProgressBar pbEmb;
		private System.Windows.Forms.Button butSaveSecMp3s;
		private uint preInjDataSize       = 0;
		private System.Windows.Forms.SaveFileDialog sfd;
		private System.Windows.Forms.Panel panChooseMP3Ex;
		private System.Windows.Forms.Label labChooseMP3ExInfo;
		private System.Windows.Forms.Button butDelMP3Ex;
		private System.Windows.Forms.Button butAddMP3Ex;
		private System.Windows.Forms.ListView lvMP3Ex;
		private utils.MP3Trojan injector  = null;
		private utils.MP3Trojan extractor = null;
		private System.Windows.Forms.ProgressBar pbExEnd;
		private System.Windows.Forms.Label labExtEnd;
		private System.Windows.Forms.Panel panExtEnd;
		private System.Windows.Forms.Button butSaveHiddens;
		private string[] strMp3sToEx = null;

		/// <summary>
		/// Short for Strings.queryText()
		/// Where are the macros in C# :-)
		/// </summary>
		/// <param name="testid"></param>
		private string T( string textid )
		{
			return Strings.queryText( textid );
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="langFilePath">entire path to our xml language file</param>
		public MainFrame( string langFilePath )
		{
			//
			// init lanugage queriing system
			//
			Strings = new utils.TextEnum( langFilePath );

			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// TODO: Add any constructor code after InitializeComponent call
			//
			llabMoreMp3s.ForeColor = System.Drawing.Color.Red;

			// insert main dlg title
			this.Text = stdTitle;

			// define panel arrangement
			tourList = new Panel[][] {
				new Panel[] {panWelcome, panWhatToDo, panChooseFiles, panChooseMP3Emb, panPassword, panEmbEnd}, // embed tour
				new Panel[] {panWelcome, panWhatToDo, panChooseMP3Ex, panPassword, panExtEnd} // extract tour
			};

			//
			// insert text into GUI controls
			//
			// available languages to combobox
			string[] langs = Strings.getAvailableLanuages();
			for ( int i = 0; i < langs.Length; i++ )
				cbLangs.Items.Add( langs[i] );
			this.cbLangs.Text = currentLanguage;

			//
			// some component-specific setups
			//
			linkLabel1.Links.Add( 0, -1, "http://sistemo.has.it" );
		}

		/// <summary>
		/// Returns strings of format:
		/// X KB  Y Byte
		/// </summary>
		/// <param name="cb"></param>
		/// <returns></returns>
		private string getSizeString( long cb )
		{
			string outStr = "";

			long bytes        = cb % 1000;
			long kbytestmp    = cb - bytes;

			outStr += String.Format( "{0} Bytes", bytes );

			long kbytesonly   = kbytestmp % 1000000;
			long mbytesonly   = kbytestmp - kbytesonly;

			if ( mbytesonly != 0 )
				outStr = String.Format( "{0} MB  {1} KB  ",
					mbytesonly / 1000000, kbytesonly / 1000 ) + outStr;
			else if ( kbytesonly != 0 )
				outStr = String.Format( "{0} KB  ", kbytesonly / 1000 ) + outStr;

			return outStr; // OK
		}

		private bool endOfTour()
		{
			return this.tourIndex == tourList[bExtract].Length-1 ? true : false;
		}

		private void initCurPanel()
		{
			//
			// generic
			//
			butBack.Text          = "<<" + Strings.queryText( "BACK" );
			butBack.Enabled       = true;
			butNext.Text          = Strings.queryText( "NEXT" ) + ">>";
			butNext.Enabled       = true;
			butCancel.Text        = Strings.queryText( endOfTour() ? "END" : "CANCEL" );
			butCancel.Enabled     = true;

			// are we at START or END of the tour ?
			butBack.Enabled = this.tourIndex == 0 ? false : true;
			butNext.Enabled = !endOfTour();

			//
			// maybe add current step
			if ( tourIndex > 1 )
			{
				// add step index in left bottom corner
				string max = bExtract == 1 ? "3" : "4";
				labStep.Text = Strings.queryText( "STEP" ) + " " +
					(tourIndex-1).ToString() + " " + T("OF") + " " + max;

				// add type of action in title
				Text = stdTitle + "  - " + Strings.queryText( bExtract == 1 ? "EXTRACTDATA" : "EMBEDDATA" );
			}
			else
			{
				labStep.Text = "";
				this.Text    = stdTitle;
			}

			//
			// global panel setups
			//
			switch( this.tourIndex )
			{
				case 0: // welcome panel
					labChooseLang.Text    = Strings.queryText( "CHOOSELANG" ) + ":";
					labWelcome.Text       = Strings.queryText( "WELCOME" );
					break;

				case 1: // whattodo panel
					radEmbed.Text         = Strings.queryText( "EMBEDDATA" );
					radExtract.Text       = Strings.queryText( "EXTRACTDATA" );
					break;
			}

			//
			// action specific panel inits
			//
			if ( bExtract == 1 )
				// extracting panels
				switch( tourIndex )
				{
					case 2: // choosemp3ex panel
						labChooseMP3ExInfo.Text   = T("MP34EX");
						butAddMP3Ex.Text          = T("ADD");
						butDelMP3Ex.Text          = T("DEL");

						lvMP3Ex.Columns.Clear();
						lvMP3Ex.Columns.Add( T("FILENAME"), 90, HorizontalAlignment.Left );
						lvMP3Ex.Columns.Add( "Id", 70, HorizontalAlignment.Center );
						lvMP3Ex.Columns.Add( T("ELINDEX"), 80, HorizontalAlignment.Center );
						lvMP3Ex.Columns.Add( T("ELTOTAL"), 80, HorizontalAlignment.Center );
						break;

					case 3: // password panel
						labEnterPW.Text       = Strings.queryText( "ENTERPW" );
						labPW1.Text           = Strings.queryText( "PASSWORD" ) + ":";
						labPW2.Text           = Strings.queryText( "AGREEMENT" ) + ":";
						txtPW1.Text           = "";
						txtPW2.Text           = "";
						break;

					case 4: // extraction end
						labExtEnd.Text         = T( "EXTEND" );
						butSaveHiddens.Text    = T( "SAVE" );
						butSaveHiddens.Enabled = false;

						extractDataInMP3();
						break;
				}
			else
				// embedding panels
				switch( tourIndex )
				{
					case 2: // choosefiles panel
						labChooseFiles.Text    = Strings.queryText( "FILES2EMBED" );
						butAddSecret.Text      = Strings.queryText( "ADD" );
						butDelSecret.Text      = Strings.queryText( "DEL" );
						labTotSecrets.Text     = Strings.queryText( "TOTALSIZE" ) + ":";
						labTotSecretFSize.Text = getSizeString( cbSecretFiles );

						lvSecretFiles.Columns.Clear();
						lvSecretFiles.Columns.Add( Strings.queryText( "FILENAME" ), 90, HorizontalAlignment.Left );
						lvSecretFiles.Columns.Add( Strings.queryText( "FILESIZE" ), 80, HorizontalAlignment.Center );
						lvSecretFiles.Columns.Add( Strings.queryText( "FILEPATH" ), 200, HorizontalAlignment.Left );
						break;

					case 3: // choosemp3filesemb panel
						butAddMP3Emb.Text    = Strings.queryText( "ADD" );
						butDelMP3Emb.Text    = Strings.queryText( "DEL" );
						llabMoreMp3s.Text    = Strings.queryText( "MOREMP3s" );
						labData2Hide.Text    = Strings.queryText( "DATA2HIDE" ) + ":";
						labChooseMP3Emb.Text = Strings.queryText( "MP3s4EMBED" );
						long num = Math.Max(preInjDataSize-cbHidable,0);
						labSize2Hide.Text    = getSizeString( num );
						llabMoreMp3s.Visible = num==0 ? false : true;

						lvMP3sEmb.Columns.Clear();
						lvMP3sEmb.Columns.Add( Strings.queryText( "FILENAME" ), 90, HorizontalAlignment.Left );
						lvMP3sEmb.Columns.Add( Strings.queryText( "LENGTH" ), 80, HorizontalAlignment.Center );
						lvMP3sEmb.Columns.Add( Strings.queryText( "BITRATE" ), 80, HorizontalAlignment.Center );
						lvMP3sEmb.Columns.Add( Strings.queryText( "FREQUENCY" ), 80, HorizontalAlignment.Center );
						lvMP3sEmb.Columns.Add( Strings.queryText( "FILEPATH" ), 200, HorizontalAlignment.Left );
						break;

					case 4: // password panel (embed)
						labEnterPW.Text       = Strings.queryText( "ENTERPW" );
						labPW1.Text           = Strings.queryText( "PASSWORD" ) + ":";
						labPW2.Text           = Strings.queryText( "AGREEMENT" ) + ":";
						txtPW1.Text           = "";
						txtPW2.Text           = "";
						break;

					case 5: // embend panel
						labEmbEnd.Text         = Strings.queryText( "EMBEND" );
						butSaveSecMp3s.Text    = Strings.queryText( "SAVE" );
						butSaveSecMp3s.Enabled = false;

						injectDataInMP3();

						break;
				}
		}

		private void injectDataInMP3()
		{
			// collect secret files
			String[] secFiles = new string[lvSecretFiles.Items.Count];
			int i = 0;
			foreach( ListViewItem item in lvSecretFiles.Items )
			{
				secFiles[i++] = Path.Combine( item.SubItems[2].Text, item.SubItems[0].Text );
			}

			// collect MP3 files
			i = 0;
			string[] mp3Files = new string[lvMP3sEmb.Items.Count];
			foreach( ListViewItem item in lvMP3sEmb.Items )
			{
				mp3Files[i++] = Path.Combine( item.SubItems[4].Text, item.SubItems[0].Text );
			}

			// get password
			string pw = txtPW1.Text;

            //
			// prepare the whole data
			//
			injector = new utils.MP3Trojan( mp3Files, secFiles );

			injector.prepareInjection(
				pw,
				new utils.MP3Trojan.prepareInjCallback(prepareInjectionProgress) );

			injector.beginMP3Stamping();

			// enable save button
			butSaveSecMp3s.Enabled = true;
		}

		private void extractDataInMP3()
		{
			// get password
			string pw = txtPW1.Text;

			//
			// prepare the whole data
			//
			extractor = new utils.MP3Trojan( strMp3sToEx, null );

			bool b = extractor.prepareExtraction(
				pw,
				new utils.MP3Trojan.prepareInjCallback(prepareExtractionProcess) );
			if ( !b )
			{
				errorMsgId( "BADPW" );
				return; // ERR
			}

			extractor.beginHiddenStamping();

			// enable save button
			butSaveHiddens.Enabled = true;
		}

		private void prepareInjectionProgress( int curStep, int ofSteps )
		{
			pbEmb.Minimum = 1;
			pbEmb.Maximum = ofSteps;
			pbEmb.Value   = curStep;
		}

		private void prepareExtractionProcess( int curStep, int ofSteps )
		{
			pbExEnd.Minimum  = 1;
			pbExEnd.Maximum  = ofSteps;
			pbExEnd.Value    = curStep;
		}

		private void errorMsg( string text )
		{
			MessageBox.Show(
				this,
				text, Strings.queryText( "ERROR" ),
				MessageBoxButtons.OK, MessageBoxIcon.Error );
		}

		private void errorMsgId( string textid )
		{
			errorMsg( Strings.queryText( textid ) );
		}

		private bool verifyCurPanel()
		{
			//
			// generic
			//
			if ( tourIndex == 1 ) // choosefiles panel
			{
				if ( !(radEmbed.Checked || radExtract.Checked) )
				{
					errorMsg( Strings.queryText( "DLGINCOMPL" ) );
					return false; // ERR
				}
				bExtract = radExtract.Checked ? 1 : 0;
				return true; // OK
			}

			//
			// task specific (extract/embed)
			//
			if ( this.bExtract == 1 )
				// extracting panels
				switch( tourIndex )
				{    
					case 2: // choose extract mp3s
                        // check whether all files of a combine
						// are existing

						if ( lvMP3Ex.Items.Count == 0 )
						{
							errorMsgId( "DLGINCOMPL" );
							return false; // ERR
						}

						// get infos of the first combine item
						string firstFname     = lvMP3Ex.Items[0].Text;
						int    firstCombId    = Convert.ToInt32( lvMP3Ex.Items[0].SubItems[1].Text );
						int    firstCombTotal = Convert.ToInt32( lvMP3Ex.Items[0].SubItems[3].Text );

						// validate the rest
						if ( lvMP3Ex.Items.Count != firstCombTotal )
						{
							errorMsgId( "BADSECMP3NUM" );
							return false; // ERR
						}

						// verify wether all items of the combine are really there
						this.strMp3sToEx = new String[firstCombTotal];
						for( int i = 0; i < firstCombTotal; i++ )
							strMp3sToEx[i] = null;
                        foreach( ListViewItem item in lvMP3Ex.Items )
						{
							string fpath    = item.Text;
							int    combId   = Convert.ToInt32( item.SubItems[1].Text );
							int    curIndex = Convert.ToInt32( item.SubItems[2].Text );
							if ( combId == firstCombId )
                                strMp3sToEx[curIndex] = fpath;							
						}

						// now just check to result
						bool bAllright = true;
						for ( int i = 0; i < firstCombTotal; i++ )
							if ( strMp3sToEx[i] == null )
								bAllright = false;
						if ( !bAllright )
						{
							errorMsgId( "ALLSECSNEEDED" );
							return false; // ERR
						}
						break;

					case 3: // password panel
						if ( txtPW1.Text != txtPW2.Text )
						{
							errorMsgId( "PWMATCHERR" );
							txtPW1.Text = txtPW2.Text = "";
							return false; // ERR
						}
						break;
				}
			else
				// embedding panels
				switch( tourIndex )
				{
					case 2: // choosefiles panel
						if ( lvSecretFiles.Items.Count == 0 )
						{
							errorMsg( Strings.queryText("DLGINCOMPL") );
							return false; // ERR
						}

						// precalucate the size of the inject data
						using( FileContainerStream fcTmp = new FileContainerStream() )
						{
							// add all selected files in the file container
							foreach( ListViewItem item in lvSecretFiles.Items )
							{
								string filePath = Path.Combine(
									item.SubItems[2].Text, item.Text );
								fcTmp.AddFile( filePath );
							}

							// save fc size + 100 bytes as pre-inject-data-size
							preInjDataSize = (uint)fcTmp.Length + CF_PROLOG_SIZE_ASSUMPTION;
						}
						break;

					case 3: // choosemp3emb panel
						if ( llabMoreMp3s.Visible )
						{
							errorMsg( Strings.queryText("DLGINCOMPL") );
							return false; // ERR
						}						
						break;

					case 4: // password panel
						if ( txtPW1.Text != txtPW2.Text )
						{
							errorMsgId( "PWMATCHERR" );
							txtPW1.Text = txtPW2.Text = "";
							return false; // ERR
						}
						break;
				}

			return true; // OK - default value
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(MainFrame));
			this.pictureBox1 = new System.Windows.Forms.PictureBox();
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.groupBox2 = new System.Windows.Forms.GroupBox();
			this.butBack = new System.Windows.Forms.Button();
			this.butCancel = new System.Windows.Forms.Button();
			this.butNext = new System.Windows.Forms.Button();
			this.panWelcome = new System.Windows.Forms.Panel();
			this.label2 = new System.Windows.Forms.Label();
			this.labWelcome = new System.Windows.Forms.Label();
			this.labChooseLang = new System.Windows.Forms.Label();
			this.cbLangs = new System.Windows.Forms.ComboBox();
			this.linkLabel1 = new System.Windows.Forms.LinkLabel();
			this.label1 = new System.Windows.Forms.Label();
			this.groupBox3 = new System.Windows.Forms.GroupBox();
			this.panExtEnd = new System.Windows.Forms.Panel();
			this.butSaveHiddens = new System.Windows.Forms.Button();
			this.pbExEnd = new System.Windows.Forms.ProgressBar();
			this.labExtEnd = new System.Windows.Forms.Label();
			this.panChooseMP3Ex = new System.Windows.Forms.Panel();
			this.butDelMP3Ex = new System.Windows.Forms.Button();
			this.butAddMP3Ex = new System.Windows.Forms.Button();
			this.lvMP3Ex = new System.Windows.Forms.ListView();
			this.labChooseMP3ExInfo = new System.Windows.Forms.Label();
			this.panChooseMP3Emb = new System.Windows.Forms.Panel();
			this.llabMoreMp3s = new System.Windows.Forms.Label();
			this.labSize2Hide = new System.Windows.Forms.Label();
			this.labData2Hide = new System.Windows.Forms.Label();
			this.butDelMP3Emb = new System.Windows.Forms.Button();
			this.butAddMP3Emb = new System.Windows.Forms.Button();
			this.lvMP3sEmb = new System.Windows.Forms.ListView();
			this.labChooseMP3Emb = new System.Windows.Forms.Label();
			this.panEmbEnd = new System.Windows.Forms.Panel();
			this.butSaveSecMp3s = new System.Windows.Forms.Button();
			this.pbEmb = new System.Windows.Forms.ProgressBar();
			this.labEmbEnd = new System.Windows.Forms.Label();
			this.panPassword = new System.Windows.Forms.Panel();
			this.txtPW2 = new System.Windows.Forms.TextBox();
			this.labPW2 = new System.Windows.Forms.Label();
			this.txtPW1 = new System.Windows.Forms.TextBox();
			this.labPW1 = new System.Windows.Forms.Label();
			this.labEnterPW = new System.Windows.Forms.Label();
			this.panChooseFiles = new System.Windows.Forms.Panel();
			this.butDelSecret = new System.Windows.Forms.Button();
			this.butAddSecret = new System.Windows.Forms.Button();
			this.labTotSecretFSize = new System.Windows.Forms.Label();
			this.labTotSecrets = new System.Windows.Forms.Label();
			this.lvSecretFiles = new System.Windows.Forms.ListView();
			this.labChooseFiles = new System.Windows.Forms.Label();
			this.panWhatToDo = new System.Windows.Forms.Panel();
			this.radExtract = new System.Windows.Forms.RadioButton();
			this.radEmbed = new System.Windows.Forms.RadioButton();
			this.ofn = new System.Windows.Forms.OpenFileDialog();
			this.labStep = new System.Windows.Forms.Label();
			this.sfd = new System.Windows.Forms.SaveFileDialog();
			this.groupBox1.SuspendLayout();
			this.panWelcome.SuspendLayout();
			this.groupBox3.SuspendLayout();
			this.panExtEnd.SuspendLayout();
			this.panChooseMP3Ex.SuspendLayout();
			this.panChooseMP3Emb.SuspendLayout();
			this.panEmbEnd.SuspendLayout();
			this.panPassword.SuspendLayout();
			this.panChooseFiles.SuspendLayout();
			this.panWhatToDo.SuspendLayout();
			this.SuspendLayout();
			// 
			// pictureBox1
			// 
			this.pictureBox1.Image = ((System.Drawing.Bitmap)(resources.GetObject("pictureBox1.Image")));
			this.pictureBox1.Location = new System.Drawing.Point(10, 16);
			this.pictureBox1.Name = "pictureBox1";
			this.pictureBox1.Size = new System.Drawing.Size(136, 248);
			this.pictureBox1.TabIndex = 0;
			this.pictureBox1.TabStop = false;
			// 
			// groupBox1
			// 
			this.groupBox1.Controls.AddRange(new System.Windows.Forms.Control[] {
																					this.pictureBox1});
			this.groupBox1.Location = new System.Drawing.Point(8, 8);
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size(152, 267);
			this.groupBox1.TabIndex = 1;
			this.groupBox1.TabStop = false;
			// 
			// groupBox2
			// 
			this.groupBox2.Location = new System.Drawing.Point(16, 280);
			this.groupBox2.Name = "groupBox2";
			this.groupBox2.Size = new System.Drawing.Size(480, 8);
			this.groupBox2.TabIndex = 2;
			this.groupBox2.TabStop = false;
			// 
			// butBack
			// 
			this.butBack.Enabled = false;
			this.butBack.Location = new System.Drawing.Point(184, 296);
			this.butBack.Name = "butBack";
			this.butBack.Size = new System.Drawing.Size(88, 23);
			this.butBack.TabIndex = 3;
			this.butBack.Text = "(b)";
			this.butBack.Click += new System.EventHandler(this.butBack_Click);
			// 
			// butCancel
			// 
			this.butCancel.Location = new System.Drawing.Point(392, 296);
			this.butCancel.Name = "butCancel";
			this.butCancel.Size = new System.Drawing.Size(88, 23);
			this.butCancel.TabIndex = 4;
			this.butCancel.Text = "(c)";
			this.butCancel.Click += new System.EventHandler(this.button2_Click);
			// 
			// butNext
			// 
			this.butNext.Location = new System.Drawing.Point(288, 296);
			this.butNext.Name = "butNext";
			this.butNext.Size = new System.Drawing.Size(88, 23);
			this.butNext.TabIndex = 5;
			this.butNext.Text = "(n)";
			this.butNext.Click += new System.EventHandler(this.butNext_Click);
			// 
			// panWelcome
			// 
			this.panWelcome.Controls.AddRange(new System.Windows.Forms.Control[] {
																					 this.label2,
																					 this.labWelcome,
																					 this.labChooseLang,
																					 this.cbLangs,
																					 this.linkLabel1});
			this.panWelcome.Location = new System.Drawing.Point(8, 8);
			this.panWelcome.Name = "panWelcome";
			this.panWelcome.Size = new System.Drawing.Size(320, 256);
			this.panWelcome.TabIndex = 6;
			// 
			// label2
			// 
			this.label2.Location = new System.Drawing.Point(96, 199);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(136, 16);
			this.label2.TabIndex = 4;
			this.label2.Text = "Danilo Bzdok   2003-2004";
			// 
			// labWelcome
			// 
			this.labWelcome.Location = new System.Drawing.Point(8, 16);
			this.labWelcome.Name = "labWelcome";
			this.labWelcome.Size = new System.Drawing.Size(304, 80);
			this.labWelcome.TabIndex = 3;
			this.labWelcome.Text = "(welcome text)";
			this.labWelcome.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// labChooseLang
			// 
			this.labChooseLang.Location = new System.Drawing.Point(24, 144);
			this.labChooseLang.Name = "labChooseLang";
			this.labChooseLang.Size = new System.Drawing.Size(128, 16);
			this.labChooseLang.TabIndex = 1;
			this.labChooseLang.Text = "(choose ur lang)";
			// 
			// cbLangs
			// 
			this.cbLangs.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.cbLangs.Location = new System.Drawing.Point(176, 140);
			this.cbLangs.Name = "cbLangs";
			this.cbLangs.Size = new System.Drawing.Size(128, 21);
			this.cbLangs.Sorted = true;
			this.cbLangs.TabIndex = 2;
			this.cbLangs.SelectedIndexChanged += new System.EventHandler(this.cbLangs_SelectedIndexChanged);
			// 
			// linkLabel1
			// 
			this.linkLabel1.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
			this.linkLabel1.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.linkLabel1.LinkBehavior = System.Windows.Forms.LinkBehavior.HoverUnderline;
			this.linkLabel1.Location = new System.Drawing.Point(107, 218);
			this.linkLabel1.Name = "linkLabel1";
			this.linkLabel1.Size = new System.Drawing.Size(112, 16);
			this.linkLabel1.TabIndex = 0;
			this.linkLabel1.TabStop = true;
			this.linkLabel1.Text = "http://sistemo.has.it";
			this.linkLabel1.LinkClicked += new System.Windows.Forms.LinkLabelLinkClickedEventHandler(this.linkLabel1_LinkClicked);
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(110, 117);
			this.label1.Name = "label1";
			this.label1.TabIndex = 2;
			this.label1.Text = "label1";
			// 
			// groupBox3
			// 
			this.groupBox3.Controls.AddRange(new System.Windows.Forms.Control[] {
																					this.panWelcome,
																					this.panExtEnd,
																					this.panChooseMP3Ex,
																					this.panChooseMP3Emb,
																					this.panEmbEnd,
																					this.panPassword,
																					this.panChooseFiles,
																					this.panWhatToDo});
			this.groupBox3.Location = new System.Drawing.Point(168, 8);
			this.groupBox3.Name = "groupBox3";
			this.groupBox3.Size = new System.Drawing.Size(336, 268);
			this.groupBox3.TabIndex = 3;
			this.groupBox3.TabStop = false;
			// 
			// panExtEnd
			// 
			this.panExtEnd.Controls.AddRange(new System.Windows.Forms.Control[] {
																					this.butSaveHiddens,
																					this.pbExEnd,
																					this.labExtEnd});
			this.panExtEnd.Location = new System.Drawing.Point(8, 8);
			this.panExtEnd.Name = "panExtEnd";
			this.panExtEnd.Size = new System.Drawing.Size(320, 256);
			this.panExtEnd.TabIndex = 3;
			this.panExtEnd.Visible = false;
			// 
			// butSaveHiddens
			// 
			this.butSaveHiddens.Location = new System.Drawing.Point(96, 168);
			this.butSaveHiddens.Name = "butSaveHiddens";
			this.butSaveHiddens.Size = new System.Drawing.Size(136, 24);
			this.butSaveHiddens.TabIndex = 5;
			this.butSaveHiddens.Text = "(save)";
			this.butSaveHiddens.Click += new System.EventHandler(this.butSaveHiddens_Click);
			// 
			// pbExEnd
			// 
			this.pbExEnd.Location = new System.Drawing.Point(32, 128);
			this.pbExEnd.Name = "pbExEnd";
			this.pbExEnd.Size = new System.Drawing.Size(256, 14);
			this.pbExEnd.TabIndex = 4;
			// 
			// labExtEnd
			// 
			this.labExtEnd.Location = new System.Drawing.Point(24, 16);
			this.labExtEnd.Name = "labExtEnd";
			this.labExtEnd.Size = new System.Drawing.Size(280, 88);
			this.labExtEnd.TabIndex = 3;
			this.labExtEnd.Text = "(lalala)";
			// 
			// panChooseMP3Ex
			// 
			this.panChooseMP3Ex.Controls.AddRange(new System.Windows.Forms.Control[] {
																						 this.butDelMP3Ex,
																						 this.butAddMP3Ex,
																						 this.lvMP3Ex,
																						 this.labChooseMP3ExInfo});
			this.panChooseMP3Ex.Location = new System.Drawing.Point(8, 8);
			this.panChooseMP3Ex.Name = "panChooseMP3Ex";
			this.panChooseMP3Ex.Size = new System.Drawing.Size(320, 256);
			this.panChooseMP3Ex.TabIndex = 15;
			this.panChooseMP3Ex.Visible = false;
			// 
			// butDelMP3Ex
			// 
			this.butDelMP3Ex.Enabled = false;
			this.butDelMP3Ex.Location = new System.Drawing.Point(216, 216);
			this.butDelMP3Ex.Name = "butDelMP3Ex";
			this.butDelMP3Ex.Size = new System.Drawing.Size(88, 23);
			this.butDelMP3Ex.TabIndex = 12;
			this.butDelMP3Ex.Text = "(del)";
			this.butDelMP3Ex.Click += new System.EventHandler(this.butDelMP3Ex_Click);
			// 
			// butAddMP3Ex
			// 
			this.butAddMP3Ex.Location = new System.Drawing.Point(16, 216);
			this.butAddMP3Ex.Name = "butAddMP3Ex";
			this.butAddMP3Ex.Size = new System.Drawing.Size(88, 23);
			this.butAddMP3Ex.TabIndex = 11;
			this.butAddMP3Ex.Text = "(add)";
			this.butAddMP3Ex.Click += new System.EventHandler(this.butAddMP3Ex_Click);
			// 
			// lvMP3Ex
			// 
			this.lvMP3Ex.FullRowSelect = true;
			this.lvMP3Ex.Location = new System.Drawing.Point(16, 88);
			this.lvMP3Ex.MultiSelect = false;
			this.lvMP3Ex.Name = "lvMP3Ex";
			this.lvMP3Ex.Size = new System.Drawing.Size(288, 120);
			this.lvMP3Ex.TabIndex = 10;
			this.lvMP3Ex.View = System.Windows.Forms.View.Details;
			this.lvMP3Ex.SelectedIndexChanged += new System.EventHandler(this.lvMP3Ex_SelectedIndexChanged);
			// 
			// labChooseMP3ExInfo
			// 
			this.labChooseMP3ExInfo.Location = new System.Drawing.Point(24, 16);
			this.labChooseMP3ExInfo.Name = "labChooseMP3ExInfo";
			this.labChooseMP3ExInfo.Size = new System.Drawing.Size(280, 56);
			this.labChooseMP3ExInfo.TabIndex = 0;
			this.labChooseMP3ExInfo.Text = "(lalala)";
			// 
			// panChooseMP3Emb
			// 
			this.panChooseMP3Emb.Controls.AddRange(new System.Windows.Forms.Control[] {
																						  this.llabMoreMp3s,
																						  this.labSize2Hide,
																						  this.labData2Hide,
																						  this.butDelMP3Emb,
																						  this.butAddMP3Emb,
																						  this.lvMP3sEmb,
																						  this.labChooseMP3Emb});
			this.panChooseMP3Emb.Location = new System.Drawing.Point(8, 8);
			this.panChooseMP3Emb.Name = "panChooseMP3Emb";
			this.panChooseMP3Emb.Size = new System.Drawing.Size(320, 256);
			this.panChooseMP3Emb.TabIndex = 6;
			this.panChooseMP3Emb.Visible = false;
			// 
			// llabMoreMp3s
			// 
			this.llabMoreMp3s.Location = new System.Drawing.Point(8, 232);
			this.llabMoreMp3s.Name = "llabMoreMp3s";
			this.llabMoreMp3s.Size = new System.Drawing.Size(304, 18);
			this.llabMoreMp3s.TabIndex = 13;
			this.llabMoreMp3s.Text = "(more Mp3s)";
			this.llabMoreMp3s.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// labSize2Hide
			// 
			this.labSize2Hide.Location = new System.Drawing.Point(168, 216);
			this.labSize2Hide.Name = "labSize2Hide";
			this.labSize2Hide.Size = new System.Drawing.Size(136, 16);
			this.labSize2Hide.TabIndex = 12;
			this.labSize2Hide.Text = "(size)";
			this.labSize2Hide.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			// 
			// labData2Hide
			// 
			this.labData2Hide.Location = new System.Drawing.Point(16, 216);
			this.labData2Hide.Name = "labData2Hide";
			this.labData2Hide.Size = new System.Drawing.Size(152, 16);
			this.labData2Hide.TabIndex = 11;
			this.labData2Hide.Text = "(data to hide)";
			// 
			// butDelMP3Emb
			// 
			this.butDelMP3Emb.Enabled = false;
			this.butDelMP3Emb.Location = new System.Drawing.Point(216, 184);
			this.butDelMP3Emb.Name = "butDelMP3Emb";
			this.butDelMP3Emb.Size = new System.Drawing.Size(88, 23);
			this.butDelMP3Emb.TabIndex = 9;
			this.butDelMP3Emb.Text = "(del)";
			this.butDelMP3Emb.Click += new System.EventHandler(this.butDelMP3Emb_Click);
			// 
			// butAddMP3Emb
			// 
			this.butAddMP3Emb.Location = new System.Drawing.Point(16, 184);
			this.butAddMP3Emb.Name = "butAddMP3Emb";
			this.butAddMP3Emb.Size = new System.Drawing.Size(88, 23);
			this.butAddMP3Emb.TabIndex = 8;
			this.butAddMP3Emb.Text = "(add)";
			this.butAddMP3Emb.Click += new System.EventHandler(this.butAddMP3Emb_Click);
			// 
			// lvMP3sEmb
			// 
			this.lvMP3sEmb.FullRowSelect = true;
			this.lvMP3sEmb.Location = new System.Drawing.Point(16, 80);
			this.lvMP3sEmb.MultiSelect = false;
			this.lvMP3sEmb.Name = "lvMP3sEmb";
			this.lvMP3sEmb.Size = new System.Drawing.Size(288, 97);
			this.lvMP3sEmb.TabIndex = 7;
			this.lvMP3sEmb.View = System.Windows.Forms.View.Details;
			this.lvMP3sEmb.SelectedIndexChanged += new System.EventHandler(this.lvMP3sEmb_SelectedIndexChanged);
			// 
			// labChooseMP3Emb
			// 
			this.labChooseMP3Emb.Location = new System.Drawing.Point(24, 16);
			this.labChooseMP3Emb.Name = "labChooseMP3Emb";
			this.labChooseMP3Emb.Size = new System.Drawing.Size(272, 56);
			this.labChooseMP3Emb.TabIndex = 6;
			this.labChooseMP3Emb.Text = "(choose MP3s to add secret data in)";
			// 
			// panEmbEnd
			// 
			this.panEmbEnd.Controls.AddRange(new System.Windows.Forms.Control[] {
																					this.butSaveSecMp3s,
																					this.pbEmb,
																					this.labEmbEnd});
			this.panEmbEnd.Location = new System.Drawing.Point(8, 8);
			this.panEmbEnd.Name = "panEmbEnd";
			this.panEmbEnd.Size = new System.Drawing.Size(320, 256);
			this.panEmbEnd.TabIndex = 14;
			this.panEmbEnd.Visible = false;
			// 
			// butSaveSecMp3s
			// 
			this.butSaveSecMp3s.Location = new System.Drawing.Point(88, 184);
			this.butSaveSecMp3s.Name = "butSaveSecMp3s";
			this.butSaveSecMp3s.Size = new System.Drawing.Size(136, 24);
			this.butSaveSecMp3s.TabIndex = 2;
			this.butSaveSecMp3s.Text = "(save)";
			this.butSaveSecMp3s.Click += new System.EventHandler(this.butSaveSecMp3s_Click);
			// 
			// pbEmb
			// 
			this.pbEmb.Location = new System.Drawing.Point(32, 125);
			this.pbEmb.Name = "pbEmb";
			this.pbEmb.Size = new System.Drawing.Size(256, 14);
			this.pbEmb.TabIndex = 1;
			// 
			// labEmbEnd
			// 
			this.labEmbEnd.Location = new System.Drawing.Point(24, 16);
			this.labEmbEnd.Name = "labEmbEnd";
			this.labEmbEnd.Size = new System.Drawing.Size(280, 88);
			this.labEmbEnd.TabIndex = 0;
			this.labEmbEnd.Text = "(lalala)";
			// 
			// panPassword
			// 
			this.panPassword.Controls.AddRange(new System.Windows.Forms.Control[] {
																					  this.txtPW2,
																					  this.labPW2,
																					  this.txtPW1,
																					  this.labPW1,
																					  this.labEnterPW});
			this.panPassword.Location = new System.Drawing.Point(8, 8);
			this.panPassword.Name = "panPassword";
			this.panPassword.Size = new System.Drawing.Size(320, 256);
			this.panPassword.TabIndex = 13;
			this.panPassword.Visible = false;
			// 
			// txtPW2
			// 
			this.txtPW2.Location = new System.Drawing.Point(176, 160);
			this.txtPW2.Name = "txtPW2";
			this.txtPW2.PasswordChar = '?';
			this.txtPW2.Size = new System.Drawing.Size(120, 20);
			this.txtPW2.TabIndex = 5;
			this.txtPW2.Text = "textBox1";
			// 
			// labPW2
			// 
			this.labPW2.Location = new System.Drawing.Point(24, 161);
			this.labPW2.Name = "labPW2";
			this.labPW2.Size = new System.Drawing.Size(144, 16);
			this.labPW2.TabIndex = 4;
			this.labPW2.Text = "(pw)";
			// 
			// txtPW1
			// 
			this.txtPW1.Location = new System.Drawing.Point(176, 132);
			this.txtPW1.Name = "txtPW1";
			this.txtPW1.PasswordChar = '?';
			this.txtPW1.Size = new System.Drawing.Size(120, 20);
			this.txtPW1.TabIndex = 3;
			this.txtPW1.Text = "textBox1";
			// 
			// labPW1
			// 
			this.labPW1.Location = new System.Drawing.Point(24, 137);
			this.labPW1.Name = "labPW1";
			this.labPW1.Size = new System.Drawing.Size(144, 16);
			this.labPW1.TabIndex = 2;
			this.labPW1.Text = "(pw)";
			// 
			// labEnterPW
			// 
			this.labEnterPW.Location = new System.Drawing.Point(24, 16);
			this.labEnterPW.Name = "labEnterPW";
			this.labEnterPW.Size = new System.Drawing.Size(272, 72);
			this.labEnterPW.TabIndex = 1;
			this.labEnterPW.Text = "(enter PW)";
			// 
			// panChooseFiles
			// 
			this.panChooseFiles.Controls.AddRange(new System.Windows.Forms.Control[] {
																						 this.butDelSecret,
																						 this.butAddSecret,
																						 this.labTotSecretFSize,
																						 this.labTotSecrets,
																						 this.lvSecretFiles,
																						 this.labChooseFiles});
			this.panChooseFiles.Location = new System.Drawing.Point(8, 8);
			this.panChooseFiles.Name = "panChooseFiles";
			this.panChooseFiles.Size = new System.Drawing.Size(320, 256);
			this.panChooseFiles.TabIndex = 6;
			this.panChooseFiles.Visible = false;
			// 
			// butDelSecret
			// 
			this.butDelSecret.Enabled = false;
			this.butDelSecret.Location = new System.Drawing.Point(216, 192);
			this.butDelSecret.Name = "butDelSecret";
			this.butDelSecret.Size = new System.Drawing.Size(88, 23);
			this.butDelSecret.TabIndex = 5;
			this.butDelSecret.Text = "(del)";
			this.butDelSecret.Click += new System.EventHandler(this.butDelSecret_Click);
			// 
			// butAddSecret
			// 
			this.butAddSecret.Location = new System.Drawing.Point(16, 192);
			this.butAddSecret.Name = "butAddSecret";
			this.butAddSecret.Size = new System.Drawing.Size(88, 23);
			this.butAddSecret.TabIndex = 4;
			this.butAddSecret.Text = "(add)";
			this.butAddSecret.Click += new System.EventHandler(this.butAddSecret_Click);
			// 
			// labTotSecretFSize
			// 
			this.labTotSecretFSize.Location = new System.Drawing.Point(160, 232);
			this.labTotSecretFSize.Name = "labTotSecretFSize";
			this.labTotSecretFSize.Size = new System.Drawing.Size(144, 16);
			this.labTotSecretFSize.TabIndex = 3;
			this.labTotSecretFSize.Text = "label3";
			this.labTotSecretFSize.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			// 
			// labTotSecrets
			// 
			this.labTotSecrets.Location = new System.Drawing.Point(16, 232);
			this.labTotSecrets.Name = "labTotSecrets";
			this.labTotSecrets.Size = new System.Drawing.Size(136, 16);
			this.labTotSecrets.TabIndex = 2;
			this.labTotSecrets.Text = "(total byte)";
			// 
			// lvSecretFiles
			// 
			this.lvSecretFiles.FullRowSelect = true;
			this.lvSecretFiles.Location = new System.Drawing.Point(16, 88);
			this.lvSecretFiles.MultiSelect = false;
			this.lvSecretFiles.Name = "lvSecretFiles";
			this.lvSecretFiles.Size = new System.Drawing.Size(288, 97);
			this.lvSecretFiles.TabIndex = 1;
			this.lvSecretFiles.View = System.Windows.Forms.View.Details;
			this.lvSecretFiles.SelectedIndexChanged += new System.EventHandler(this.lvSecretFiles_SelectedIndexChanged);
			// 
			// labChooseFiles
			// 
			this.labChooseFiles.Location = new System.Drawing.Point(24, 16);
			this.labChooseFiles.Name = "labChooseFiles";
			this.labChooseFiles.Size = new System.Drawing.Size(272, 64);
			this.labChooseFiles.TabIndex = 0;
			this.labChooseFiles.Text = "(choose files to embed)";
			// 
			// panWhatToDo
			// 
			this.panWhatToDo.Controls.AddRange(new System.Windows.Forms.Control[] {
																					  this.radExtract,
																					  this.radEmbed});
			this.panWhatToDo.Location = new System.Drawing.Point(8, 8);
			this.panWhatToDo.Name = "panWhatToDo";
			this.panWhatToDo.Size = new System.Drawing.Size(320, 256);
			this.panWhatToDo.TabIndex = 7;
			this.panWhatToDo.Visible = false;
			// 
			// radExtract
			// 
			this.radExtract.Font = new System.Drawing.Font("Microsoft Sans Serif", 9.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.radExtract.Location = new System.Drawing.Point(40, 144);
			this.radExtract.Name = "radExtract";
			this.radExtract.Size = new System.Drawing.Size(168, 32);
			this.radExtract.TabIndex = 1;
			this.radExtract.Text = "(extr data)";
			this.radExtract.CheckedChanged += new System.EventHandler(this.radExtract_CheckedChanged);
			// 
			// radEmbed
			// 
			this.radEmbed.Font = new System.Drawing.Font("Microsoft Sans Serif", 9.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.radEmbed.Location = new System.Drawing.Point(40, 40);
			this.radEmbed.Name = "radEmbed";
			this.radEmbed.Size = new System.Drawing.Size(168, 40);
			this.radEmbed.TabIndex = 0;
			this.radEmbed.Text = "(embed data)";
			this.radEmbed.CheckedChanged += new System.EventHandler(this.radEmbed_CheckedChanged);
			// 
			// labStep
			// 
			this.labStep.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.labStep.Location = new System.Drawing.Point(24, 298);
			this.labStep.Name = "labStep";
			this.labStep.Size = new System.Drawing.Size(114, 16);
			this.labStep.TabIndex = 6;
			// 
			// MainFrame
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(514, 328);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.labStep,
																		  this.butNext,
																		  this.butCancel,
																		  this.butBack,
																		  this.groupBox2,
																		  this.groupBox1,
																		  this.groupBox3});
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.MaximizeBox = false;
			this.Name = "MainFrame";
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
			this.Text = "(title)";
			this.Closing += new System.ComponentModel.CancelEventHandler(this.MainFrame_Closing);
			this.Closed += new System.EventHandler(this.MainFrame_Closed);
			this.groupBox1.ResumeLayout(false);
			this.panWelcome.ResumeLayout(false);
			this.groupBox3.ResumeLayout(false);
			this.panExtEnd.ResumeLayout(false);
			this.panChooseMP3Ex.ResumeLayout(false);
			this.panChooseMP3Emb.ResumeLayout(false);
			this.panEmbEnd.ResumeLayout(false);
			this.panPassword.ResumeLayout(false);
			this.panChooseFiles.ResumeLayout(false);
			this.panWhatToDo.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			string xmlPath = Path.Combine(
				Application.StartupPath, "stringpool.xml" );

			// look for stringpool.xml
			if ( !File.Exists( xmlPath ) )
			{
				MessageBox.Show(
					"Couldn't find stringpool.xml.\nPlease copy it into the same directoy as camouflage.exe.",
					"Fatal Error",
					MessageBoxButtons.OK,
					MessageBoxIcon.Error );
				return; // ERR
			}

			// show GUI
			Application.Run(new MainFrame( xmlPath ));
		}

		public new void Close()
		{
			base.Close();
		}

		private void button2_Click(object sender, System.EventArgs e)
		{
			if ( endOfTour() )
			{
				Close();
				return; // RET - don't show agreement msg
			}

			// ask whether right
			string caption = Strings.queryText( "AGREEMENT" );
			string text    = Strings.queryText( "REALLYABORT" );
			DialogResult ret = MessageBox.Show( text, caption,
				MessageBoxButtons.YesNo, MessageBoxIcon.Question );
			if ( ret == DialogResult.Yes )
				this.Close();
		}

		private void cbLangs_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			ComboBox snd = sender as ComboBox;

			this.currentLanguage = (string)snd.SelectedItem;
			Strings.setLanguage( currentLanguage );

			// repaint text
			this.initCurPanel();
		}

		private void MainFrame_Closed(object sender, System.EventArgs e)
		{
		}

		private void MainFrame_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			/*
			for ( int i = 80; i > 0; i-= 20 )
			{
				this.Opacity = i;
				Thread.Sleep( 500 );
			}
			*/		
		}

		private void butNext_Click(object sender, System.EventArgs e)
		{
			if ( verifyCurPanel() )
			{
				// ... all panel fields are filled in correct

				this.tourList[bExtract][tourIndex].Visible = false;
				tourIndex++;
				this.tourList[bExtract][tourIndex].Visible = true;
				initCurPanel();
			}
		}

		private void butBack_Click(object sender, System.EventArgs e)
		{
			this.tourList[bExtract][tourIndex].Visible = false;
			this.tourList[bExtract][--tourIndex].Visible = true;
			initCurPanel();
		}

		private void radEmbed_CheckedChanged(object sender, System.EventArgs e)
		{
			this.bExtract = 0;
		}

		private void radExtract_CheckedChanged(object sender, System.EventArgs e)
		{
			this.bExtract = 1;
		}

		private void prepareOfn( string filter )
		{
			ofn.Title     = Strings.queryText( "ADD" );
			ofn.FileName  = "";
			ofn.Filter    = filter;
		}

		private void prepareSfd()
		{
			sfd.Title      = Strings.queryText( "SAVE" );
			sfd.FileName   = "";
			sfd.Filter     = Strings.queryText( "ANYFILE" ) + "|*.*";
		}

		private void butAddSecret_Click(object sender, System.EventArgs e)
		{
			prepareOfn( Strings.queryText( "ANYFILE" ) + "|*.*" );

			if ( ofn.ShowDialog( this ) == DialogResult.OK )
			{
				// get file size and check whether we've read access
				long cb;
				if ( !utils.FileUtilities.getFileSize(ofn.FileName, out cb) )
				{
					errorMsgId( "FILEACCERR" );
					return;
				}

				// add fname, fsize, path in LV
				string[] items = new String[] {
					Path.GetFileName( ofn.FileName ),
					cb.ToString(),
					Path.GetDirectoryName( ofn.FileName )
			    };
				ListViewItem newitem = new ListViewItem( items );
                lvSecretFiles.Items.Add( newitem );

				// update total size
				this.cbSecretFiles      += cb;
				labTotSecretFSize.Text   = getSizeString( cbSecretFiles );
			}
		}

		private void butDelSecret_Click(object sender, System.EventArgs e)
		{
			// subtract file size from total
			long cb                  = Convert.ToUInt32(
				lvSecretFiles.SelectedItems[0].SubItems[1].Text );
            cbSecretFiles           -= cb;
			labTotSecretFSize.Text	 = getSizeString( cbSecretFiles );

			// delete
			lvSecretFiles.Items.RemoveAt( lvSecretFiles.SelectedItems[0].Index );
		}

		private void lvSecretFiles_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			butDelSecret.Enabled = lvSecretFiles.SelectedItems.Count == 0 ? false : true;
		}

		private void linkLabel1_LinkClicked(object sender, System.Windows.Forms.LinkLabelLinkClickedEventArgs e)
		{
			linkLabel1.Links[linkLabel1.Links.IndexOf(e.Link)].Visited = true;
			// run default Internet Browser
			try{ System.Diagnostics.Process.Start( e.Link.LinkData.ToString() ); }
			catch( Exception ecp )
			{
				// TMP !!
				errorMsg( ecp.ToString() );
			}
		}

		private string getMP3AnyFileFilter()
		{
			return Strings.queryText( "MP3FILTER" ) + "|*.MP3|" +
				Strings.queryText( "ANYFILE" ) + "|*.*";
		}

		private void butAddMP3Emb_Click(object sender, System.EventArgs e)
		{
			prepareOfn( getMP3AnyFileFilter() );

			if ( ofn.ShowDialog() == DialogResult.OK )
			{
				// get inject size
				MP3Tools.MP3Info info;
				uint injBytes = MP3Trojan.getInjectSize( ofn.FileName, out info );
				if ( injBytes == uint.MaxValue )
				{
					errorMsgId( "MP3ACCERR" );
					return; // ERR
				}

				// insert Mp3 in LV
				String[] rows = new String[] {
												 Path.GetFileName( ofn.FileName ),
												 String.Format( "{0}:{1}", info.totalSeconds/60, info.totalSeconds%60 ),
												 String.Format( "{0} Bkps", info.BitrateKbps ),
												 String.Format( "{0} Hz", info.FrequencyHz ),
												 Path.GetDirectoryName( ofn.FileName )
											 };
				ListViewItem item = new ListViewItem( rows );
				lvMP3sEmb.Items.Add( item );
				
				// re-calc size of not already injectable data
				cbHidable           += injBytes;
				long num             = Math.Max( preInjDataSize - cbHidable, 0 );
				labSize2Hide.Text    = getSizeString( (long)num );
				llabMoreMp3s.Visible = num==0 ? false : true;
			}
		}

		private void butDelMP3Emb_Click(object sender, System.EventArgs e)
		{
			ListView lv = lvMP3sEmb;

			// re-calc size of not already injected data
			string mp3Path = Path.Combine( lv.SelectedItems[0].SubItems[4].Text,
				lv.SelectedItems[0].SubItems[0].Text );
			cbHidable  -= MP3Trojan.getInjectSize( mp3Path );

			long num             = Math.Max( preInjDataSize-cbHidable, 0 );
			labSize2Hide.Text    = getSizeString( num );
            llabMoreMp3s.Visible = num==0 ? false : true;

			// delete selected item from list
			lv.Items.RemoveAt( lv.SelectedItems[0].Index );
		}

		private void lvMP3sEmb_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			butDelMP3Emb.Enabled = (sender as ListView).SelectedItems.Count==0 ? false : true;		
		}

		private void butSaveSecMp3s_Click(object sender, System.EventArgs e)
		{
			// init SaveFileDialog
			prepareSfd();

			//
			// let user save all mp3 separately
			//	
			sfd.InitialDirectory  = Path.GetDirectoryName( injector.mp3List[0] );
			for( int i = 0; i < injector.mp3List.Length; i++ )
			{
				// get file name for current mp3
				sfd.FileName         = Path.GetFileName( injector.mp3List[i] );
				if ( sfd.ShowDialog() != DialogResult.OK )
					break;
                
				injector.stampNextMP3( sfd.FileName );

				if ( i == 0 )
					sfd.InitialDirectory = ".";
			}

			// disable save-button
			butSaveSecMp3s.Enabled = false;
		}

		private void butAddMP3Ex_Click(object sender, System.EventArgs e)
		{
			prepareOfn( getMP3AnyFileFilter() );

			if ( ofn.ShowDialog() == DialogResult.OK )
			{
				// obtain secret MP3 infos
				MP3Trojan.SecretMP3File secInfo = MP3Trojan.getSecretMP3Info( ofn.FileName );
				if ( secInfo == null )
				{
					errorMsgId( "NOSECMP3" );
					return; // ERR
				}

				// add relevant infos
				string[] items = new String[] {
					ofn.FileName,
					secInfo.combineId.ToString(),
					secInfo.index.ToString(),
					secInfo.totalElements.ToString()
											  };
				ListViewItem newitem = new ListViewItem( items );
				lvMP3Ex.Items.Add( newitem );
			}
		}

		private void lvMP3Ex_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			butDelMP3Ex.Enabled = (sender as ListView).SelectedItems.Count==0 ? false : true;
		}

		private void butDelMP3Ex_Click(object sender, System.EventArgs e)
		{
			ListView lv = lvMP3Ex;
			lv.Items.RemoveAt( lv.SelectedItems[0].Index );		
		}

		private void butSaveHiddens_Click(object sender, System.EventArgs e)
		{
			FileContainerStream container = this.extractor.extractContainer;

			//
			// let user save all files from the container separately
			prepareSfd();
			sfd.InitialDirectory = ".";
			
			//
			// it's possible that the encrypted stream was decrypted
			// successfully but isn't a valid FileContainerStream because
			// of an invalid password
			// -> let's test it
			bool b = container.BeginEntryReading();
			if ( !b )
			{
				errorMsgId( "BADECBLOCK" );
				return; // ERR
			}

			string         fname;
			MemoryStream   msBody;
			while( container.ReadNextEntry( out fname, out msBody ) )
			{
				sfd.FileName = fname;
				if ( sfd.ShowDialog() == DialogResult.Cancel )
					break; // Stop saving

				// save current file
				if ( !utils.FileUtilities.byteChainToFile( sfd.FileName, msBody.ToArray() ) )
				{
					errorMsgId( "FILECRTERR" );
					continue; // ERR - try next file
				}
			}
			butSaveHiddens.Enabled = false;
		}
	}
}
